!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !MODULE: land_mercury_mod
!
! !DESCRIPTION: Module LAND\_MERCURY\_MOD contains variables and routines for
! the land emissions for the GEOS-Chem mercury simulation.
!\\
!\\
! !INTERFACE:
!
      MODULE LAND_MERCURY_MOD
!
! !USES:
!
      USE HCO_ERROR_MOD    ! For real precisions (hp)
      USE PRECISION_MOD    ! For GEOS-Chem Precision (fp, f4, f8)

      IMPLICIT NONE
      PRIVATE
!
! !PUBLIC MEMBER FUNCTIONS:
!
      PUBLIC :: BIOMASSHG
      PUBLIC :: VEGEMIS
      PUBLIC :: SOILEMIS
      PUBLIC :: LAND_MERCURY_FLUX
      PUBLIC :: GTMM_DR
      PUBLIC :: SNOWPACK_MERCURY_FLUX
      PUBLIC :: INIT_LAND_MERCURY
      PUBLIC :: CLEANUP_LAND_MERCURY
!
! !REVISION HISTORY:
!  02 Jun 2010 - N. E. Selin, C. Carouge - Initial version
!  02 Jun 2010 - C. Carouge  - Group all land emissions routine for mercury 
!                              into this new module.
!  13 Aug 2010 - R. Yantosca - Added modifications for MERRA
!  25 Aug 2010 - R. Yantosca - Treat MERRA in same way as GEOS-5
!  30 Aug 2010 - R. Yantosca - Added more ProTeX headers
!  12 Apr 2011 - J. Fisher   - Add missing code from Holmes 2010
!  08 Feb 2012 - R. Yantosca - Treat GEOS-5.7.x in the same way as MERRA
!  01 Mar 2012 - R. Yantosca - Now reference new grid_mod.F90
!  11 Apr 2012 - R. Yantosca - Now reference new modis_lai_mod.F90
!  20 Aug 2013 - R. Yantosca - Removed "define.h", this is now obsolete
!  17 Nov 2014 - M. Yannetti - Added PRECISION_MOD
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !PRIVATE TYPES:
!
      ! Plant transpiration rate [m/s]
      REAL(fp),  ALLOCATABLE :: TRANSP(:,:)

      CONTAINS
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: land_mercury_flux
!
! !DESCRIPTION: Subroutine LAND\_MERCURY\_FLUX calculates emissions of Hg(0) 
!  from prompt recycling of previously deposited mercury to land, in [kg/s].  
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE LAND_MERCURY_FLUX( LFLUX, LHGSNOW, State_Met )
!
! !USES:
!
      USE DAO_MOD,            ONLY : IS_ICE, IS_LAND
      USE DEPO_MERCURY_MOD,   ONLY : WD_HGP, WD_HG2, DD_HGP, DD_HG2
      USE GIGC_State_Met_Mod, ONLY : MetState
      USE TIME_MOD,           ONLY : GET_TS_EMIS
      USE TRACERID_MOD,       ONLY : ID_Hg0,          N_Hg_CATS

      USE CMN_SIZE_MOD             ! Size parameters
!
! !INPUT PARAMETERS:
!
      LOGICAL,        INTENT(IN)  :: LHGSNOW     ! Use Hg0 from snow?
      TYPE(MetState), INTENT(IN)  :: State_Met   ! Meteorology State object
!
! !OUTPUT PARAMETERS:
!
      REAL(fp),         INTENT(OUT) :: LFLUX(IIPAR,JJPAR,N_Hg_CATS) ! Hg0 flux
                                                                  !  [kg/s]
!
! !REVISION HISTORY:
!  30 Aug 2010 - N. E. Selin, C. Holmes, B. Corbitt - Initial version
!  (1 ) Now uses SNOWMAS from DAO_MOD for compatibility with GEOS-5.
!       (eds 7/30/08)
!  (2 ) Now includes REEMFRAC in parallelization; previous versions may have
!       overwritten variable. (cdh, eds 7/30/08)
!  (3 ) Now also reemit Hg(0) from ice surfaces, including sea ice 
!       (cdh, 8/19/08)
!  13 Aug 2010 - R. Yantosca - Add modifications for MERRA
!  25 Aug 2010 - R. Yantosca - Treat MERRA in same way as GEOS-5
!  26 Apr 2011 - J. Fisher   - Use MERRA land fraction information
!  12 Apr 2011 - J. Fisher   - Add missing code from Holmes 2010
!  08 Feb 2012 - R. Yantosca - Treat GEOS-5.7.x in the same way as MERRA
!  09 Nov 2012 - M. Payer    - Replaced all met field arrays with State_Met
!                              derived type object
!  26 Sep 2013 - R. Yantosca - Renamed GEOS_57 Cpp switch to GEOS_FP
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!  
      REAL(fp)                :: DTSRCE, REEMFRAC
      REAL(fp)                :: FRAC_SNOW_OR_ICE, FRAC_SNOWFREE_LAND 
      LOGICAL               :: IS_SNOW_OR_ICE, IS_LAND_OR_ICE 
      REAL(fp), PARAMETER     :: SEC_PER_YR = 365.25e+0_fp * 86400e+0_fp
      INTEGER               :: I,      J,      NN

      !=================================================================
      ! LAND_MERCURY_FLUX begins here!
      !=================================================================

      ! Emission timestep [s]
      DTSRCE = GET_TS_EMIS() * 60e+0_fp     

!$OMP PARALLEL DO 
!$OMP+DEFAULT( SHARED ) 
!$OMP+PRIVATE( I,              J,                NN                 )
!$OMP+PRIVATE( REEMFRAC,       FRAC_SNOW_OR_ICE, FRAC_SNOWFREE_LAND )  
!$OMP+PRIVATE( IS_SNOW_OR_ICE, IS_LAND_OR_ICE                       )
      DO J  = 1, JJPAR
      DO I  = 1, IIPAR
      DO NN = 1, N_Hg_CATS
    

         ! For versions prior to MERRA, have to set snow/land fractions
         ! to 1 depending which type of box we are in.
         ! Assume here default of land box with no snow. We will reset
         ! fractions for MERRA here and for prior versions snow/ice boxes
         ! below (jaf, 4/26/11)
         FRAC_SNOW_OR_ICE    = 0e+0_fp
         FRAC_SNOWFREE_LAND  = 1e+0_fp

#if   defined( MERRA ) || defined( GEOS_FP )
         ! Distinguish between ice/snow and snow-free land
         FRAC_SNOW_OR_ICE    = MIN( State_Met%FRSNO(I,J)     +
     &                              State_Met%FRSEAICE(I,J)  + 
     &                              State_Met%FRLANDIC(I,J), 1e+0_fp)
         FRAC_SNOWFREE_LAND  = MAX( State_Met%FRLAND(I,J)    - 
     &                              State_Met%FRSNO(I,J),    0e+0_fp)

         ! Set this to FALSE in MERRA to avoid overwriting fractions
         IS_SNOW_OR_ICE      = .FALSE.
         IS_LAND_OR_ICE      = (( FRAC_SNOWFREE_LAND > 0e+0_fp ) .OR.
     &                          ( FRAC_SNOW_OR_ICE   > 0e+0_fp ))

#elif defined( GEOS_5 )
         ! GEOS5 snow height (water equivalent) in mm. (Docs incorrectly say m)
         IS_SNOW_OR_ICE = (( IS_ICE ( I, J, State_Met     ))  .OR. 
     &                     ( IS_LAND( I, J, State_Met     )   .AND.
     &                       State_Met%SNOMAS(I,J) > 10e+0_fp ))
         IS_LAND_OR_ICE = (( IS_LAND( I, J, State_Met     ))  .OR.
     &                     ( IS_ICE ( I, J, State_Met     )))

#else
         ! GEOS1-4 snow heigt (water equivalent) in mm
         IS_SNOW_OR_ICE = (( IS_ICE ( I, J, State_Met     ))  .OR.
     &                     ( IS_LAND( I, J, State_Met     )   .AND. 
     &                       State_Met%SNOW(I,J)   > 10e+0_fp ))
         IS_LAND_OR_ICE = (( IS_LAND( I, J, State_Met     ))  .OR.
     &                     ( IS_ICE ( I, J, State_Met     )))
#endif

         ! Reset land fractions if we are in a version prior to MERRA
         ! and also in an ice- or snow-covered box (jaf, 4/26/11)
         IF ( IS_SNOW_OR_ICE ) THEN
            FRAC_SNOW_OR_ICE = 1e+0_fp
            FRAC_SNOWFREE_LAND  = 0e+0_fp
         ENDIF

         ! If snow or ice on the ground, reemission fraction is 0.6,
         ! otherwise 0.2
         IF ( IS_LAND_OR_ICE ) THEN

            IF (LHGSNOW) THEN
               REEMFRAC = 0.2e+0_fp * FRAC_SNOWFREE_LAND
            ELSE
               REEMFRAC = 0.2e+0_fp * FRAC_SNOWFREE_LAND +
     &                    0.6e+0_fp * FRAC_SNOW_OR_ICE
            ENDIF
            
            ! Mass of emitted Hg(0), kg
            LFLUX(I,J,NN) =
     &           ( WD_HgP(I,J,NN)+
     &           WD_Hg2(I,J,NN)+
     &           DD_HgP(I,J,NN)+
     &           DD_Hg2(I,J,NN) ) * REEMFRAC
            
            ! Emission rate of Hg(0). Convert kg /timestep -> kg/s
            LFLUX(I,J,NN) = LFLUX(I,J,NN) / DTSRCE
             
         ELSE
         
            ! No flux from non-land surfaces (water, sea ice)
            LFLUX(I,J,NN) = 0e+0_fp
         
         ENDIF

      ENDDO
      ENDDO
      ENDDO
!$OMP END PARALLEL DO
     
      END SUBROUTINE LAND_MERCURY_FLUX
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: biomasshg
!
! !DESCRIPTION: Subroutine BIOMASSHG is the subroutine for Hg(0) emissions 
!  from biomass burning. These emissions are active only for present day 
!  simulations and not for preindustrial simulations.
!\\
!\\
! !INTERFACE:
! 
      SUBROUTINE BIOMASSHG( am_I_Root, Input_Opt, EHg0_bb, RC )
!
! !USES:
!     
      USE CMN_SIZE_MOD
      USE CMN_DIAG_MOD
      USE GIGC_ErrCode_Mod
      USE GIGC_Input_Opt_Mod, ONLY : OptInput

      ! HEMCO update
      USE HCO_ERROR_MOD
      USE HCO_STATE_MOD,      ONLY : HCO_STATE
      USE HCOI_GC_MAIN_MOD,   ONLY : GetHcoDiagn
      USE HCOI_GC_MAIN_MOD,   ONLY : GetHcoState
      USE ERROR_MOD,          ONLY : ERROR_STOP
!
! !INPUT PARAMETERS:
!
      LOGICAL,                INTENT(IN)  :: am_I_Root   ! Root CPU
      TYPE(OptInput),         INTENT(IN)  :: Input_Opt   ! Input Options
!
! !OUTPUT PARAMETERS:
!
      REAL(fp),  DIMENSION(:,:),INTENT(OUT) :: EHg0_bb
      INTEGER,                INTENT(OUT) :: RC          ! Success or failure?
!
! !REMARKS:
!  Emissions are based on an inventory of CO emissions from biomass burning 
!  (Duncan et al. J Geophys Res 2003), multiplied by a Hg/CO ratio in BB 
!  plumes from Franz Slemr (Poster, EGU 2006).
!                                                                             .
!  Slemr surveyed emission factors from measurements worldwide. Although his
!  best estimate was 1.5e-7 mol Hg/ mol CO, we chose the highest value
!  (2.1e-7 mol Hg/ mol CO) in the range because the simulations shown in
!  Selin et al. (GBC 2008) required large Hg(0) emissions to sustain
!  reasonable atmospheric Hg(0) concentrations. (eck, 11/13/2008)
!
! !REVISION HISTORY:
!  30 Jul 2008 - N. E. Selin, C. Holmes, B. Corbitt - Initial version 
!  12 Apr 2011 - J. Fisher   - Add missing code from Holmes 2010
!  01 Mar 2012 - R. Yantosca - Now use GET_AREA_CM2(I,J,L) from grid_mod.F90
!  15 Aug 2014 - R. Yantosca - Removed reference to IDBCO; now obsolete
!  23 Sep 2014 - C. Keller   - Now use HEMCO diagnostics
!EOP
!------------------------------------------------------------------------------
!BOC
! 
! !LOCAL VARIABLES:
!     
!      REAL(fp)                 :: DTSRCE, E_CO, AREA_CM2
      INTEGER                :: I, J

!      ! Hg molar mass, kg Hg/ mole Hg
!      REAL(fp),  PARAMETER   :: FMOL_HG     = 200.59d-3

      ! Hg/CO molar ratio in BB emissions, mol/mol
      ! emission factor 1.5e-7 molHg/molCO (Slemr et al poster EGU 2006)
      ! change emission factor to 2.1
      ! Update to Holmes et al. 2010 version (jaf, 4/11/11)
      !REAL(fp),  PARAMETER   :: BBRatio_Hg_CO = 2.1D-7
      ! Literature range is 70-240e-9. ARCTAS gives 80-130e-9 
      ! Choose 100e-9 (Holmes et al. 2010 )
      ! REAL(fp),  PARAMETER   :: BBRatio_Hg_CO = 1d-7
      !
      ! UPDATE: In HEMCO, Hg is now an additional biomass burning tracer
      ! and the emissions factors are explicitly set in the GFED3 / FINN
      ! scale factor tables (ckeller, 09/23/2014).

      ! HEMCO update
      CHARACTER(LEN= 63)       :: DgnName
      REAL(sp), POINTER        :: Ptr2D(:,:) => NULL()  
      TYPE(HCO_STATE), POINTER :: HcoState   => NULL()

      CHARACTER(LEN=255)       :: MSG
      CHARACTER(LEN=255)       :: LOC ='BIOMASSHG (land_mercury_mod.F)'
 
      !=================================================================
      ! BIOMASSHG begins here!
      !=================================================================

      ! Do biomass Hg emissions if biomass burning is on and it is a 
      ! present-day simulation (i.e. not preindustrial)
      IF ( .NOT. Input_Opt%LPREINDHG ) THEN
       
         CALL GetHcoState ( HcoState )
         IF ( .NOT. ASSOCIATED(HcoState) ) THEN
            CALL ERROR_STOP ( 'HcoState not associated!', LOC )
         ENDIF

         ! Get diagnostics from HEMCO. The HG0 biomass diagnostics
         ! is defined in hcoi_gc_diagn_mod.F90 and becomes updated
         ! by HEMCO automatically. Output unit is as specified when
         ! defining diagnostics (kg/m2/s).
         DgnName = 'BIOMASS_HG0'
         CALL GetHcoDiagn( am_I_Root, DgnName, .TRUE., RC, Ptr2D=Ptr2D )
         IF ( RC /= HCO_SUCCESS ) CALL ERROR_STOP ( DgnName, LOC )
         EHg0_bb = Ptr2D(:,:) * HcoState%Grid%AREA_M2%Val(:,:)

      ELSE

         ! No emissions for preindustrial period, or when BB is turned off.
         EHg0_bb = 0e+0_fp

      ENDIF

      ! Free pointer
      Ptr2D    => NULL()
      HcoState => NULL()

      END SUBROUTINE BIOMASSHG
!EOC
!-----------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: vegemis
!
! !DESCRIPTION: Subroutine VEGEMIS is the subroutine for Hg(0) emissions from 
!  vegetation by evapotranspiration.
!\\
!\\
! !INTERFACE:
!
      ! Bug fix: VEGEMIS shouldn't be tied to GCAP emissions
      ! (jaf, eds, 4/1/11)
      SUBROUTINE VEGEMIS( am_I_Root, Input_Opt, State_Met,
     &                    LVEGEMIS,  EHg0_dist, EHg0_vg,   RC )
!
! !USES:
!
      USE DAO_MOD,            ONLY : IS_LAND
      USE GIGC_ErrCode_Mod
      USE GIGC_Input_Opt_Mod, ONLY : OptInput
      USE GIGC_State_Met_Mod, ONLY : MetState
      USE GRID_MOD,           ONLY : GET_AREA_M2
      USE TIME_MOD,           ONLY : GET_MONTH, ITS_A_NEW_MONTH
      USE TIME_MOD,           ONLY : GET_TS_EMIS
      USE CMN_SIZE_MOD
!
! !INPUT PARAMETERS:
!
      LOGICAL,                INTENT(IN)  :: am_I_Root   ! Are we on root CPU?
      TYPE(OptInput),         INTENT(IN)  :: Input_Opt   ! Input Options object
      TYPE(MetState),         INTENT(IN)  :: State_Met   ! Met State object
      LOGICAL,                INTENT(IN)  :: LVEGEMIS    !
      REAL(fp), DIMENSION(:,:), INTENT(IN)  :: EHg0_dist   ! 

!
! !OUTPUT PARAMETERS:
!
      REAL(fp), DIMENSION(:,:), INTENT(OUT) :: EHg0_vg
      INTEGER,                INTENT(OUT) :: RC          ! Success or failure?
!
! !REMARKS:
!  Vegetation emissions are proportional to the evapotranspiration rate and the
!  soil water mercury content. We assume a constant concentration of mercury
!  in soil matter, based on the preindustrial and present-day simulations
!  described in Selin et al. (GBC 2008) and in SOILEMIS subroutine. From the
!  soil matter Hg concentration, we calculate a soil water Hg concentration in 
!  equilibrium (Allison and Allison, 2005).
!  NASA provides a climatology of evapotranspiration based on a water budget
!  model (Mintz and Walker, 1993).
!
! Calculate vegetation emissions following Xu et al (1999)
!    Fc = Ec Cw
!
!    Fc is Hg0 flux (ng m-2 s-1)
!    Ec is canopy transpiration (m s-1)
!    Cw is conc of Hg0 in surface soil water (ng m-3)
!
! Calculate Cw from the Allison and Allison (2005) equilibrium formula
!    Cw = Cs / Kd
!
!    Cs is the concentration of Hg is surface soil solids, ng/g
!    Kd is the equilibrium constant = [sorbed]/[dissolved]
!       log Kd = 3.8 L/kg -> Kd = 6310 L /kg = 6.31D-3 m3/g
!
! We assume a global mean Cs = 45 ng/g for the preindustrial period. In
! iterative simulations we redistribute this according to the deposition
! pattern while maintining the global mean. The scaling factor, EHg0_dist,
! also accounts for the anthropogenic enhancement of soil Hg in the present 
! day. 

! !REVISION HISTORY:
!  30 Aug 2010 - N. Eckley, C. Holmes, B. Corbitt - Initial version
!  01 Mar 2012 - R. Yantosca - Now use GET_AREA_M2(I,J,L) from grid_mod.F90
!  20 Jun 2014 - R. Yantosca - Now accept Input_Opt via the arg list
!EOP
!------------------------------------------------------------------------------
!BOC
! 
! !LOCAL VARIABLES:
!     
      REAL(fp)        :: DRYSOIL_HG, SOILWATER_HG, AREA_M2, VEG_EMIS
      INTEGER         :: I, J

      ! Soil Hg sorption to dissolution ratio, m3/g
      REAL(fp), PARAMETER  :: Kd = 6.31e-3_fp

      ! Preindustrial global mean soil Hg concentration, ng Hg /g dry soil
      REAL(fp), PARAMETER  :: DRYSOIL_PREIND_HG = 45e+0_fp

      !=================================================================
      ! VEGEMIS begins here!
      !=================================================================

      ! Assume success
      RC = GIGC_SUCCESS

      ! No emissions through transpiration if we use Bess' GCAP emissions
      ! Bug fix: VEGEMIS shouldn't be tied to GCAP emissions
      ! (jaf, eds, 4/1/11)
      !IF (LGCAPEMIS) THEN

         EHg0_vg = 0e+0_fp

      END SUBROUTINE VEGEMIS
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: soilemis
!
! !DESCRIPTION: Subroutine SOILEMIS is the subroutine for Hg(0) emissions 
!  from soils.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE SOILEMIS( EHg0_dist, EHg0_so, State_Met )
!
! !USES:
!
      USE DAO_MOD,            ONLY : IS_LAND
      USE GIGC_State_Met_Mod, ONLY : MetState
      USE GRID_MOD,           ONLY : GET_AREA_M2
      USE MODIS_LAI_MOD,      ONLY : ISOLAI  => GC_LAI
      USE TIME_MOD,           ONLY : GET_MONTH, ITS_A_NEW_MONTH
      USE TIME_MOD,           ONLY : GET_TS_EMIS

      USE CMN_SIZE_MOD             ! Size parameters
!
! !INPUT PARAMETERS:
!
      REAL(fp), DIMENSION(:,:), INTENT(IN)  :: EHg0_dist
      TYPE(MetState),         INTENT(IN)  :: State_Met   ! Met State object
!
! !OUTPUT PARAMETERS:
!
      REAL(fp), DIMENSION(:,:), INTENT(OUT) :: EHg0_so
!
! !REMARKS:
!  Soil emissions are a function of solar radiation at ground level 
!  (accounting for attenuation by leaf canopy) and surface temperature. 
!  The radiation dependence from Zhang et al. (2000) is multiplied by the 
!  temperature dependence from Poissant and Casimir (1998). 
!  Finally, this emission factor is multiplied by the soil mercury
!  concentration and scaled to meet the global emission total.
!
!  Comments on soil Hg concentration:
!  ----------------------------------
!  We chose the preindustrial value of 45 ng Hg /g dry soil as the mean of
!  the range quoted in Selin et al. (GBC 2008): 20-70 ng/g (Andersson, 1967; 
!  Shacklette et al., 1971; Richardson et al., 2003; Frescholtz and Gustin,
!  2004). Present-day soil concentrations are thought to be 15% greater than
!  preindustrial (Mason and Sheu 2002), but such a difference is much less
!  than the range of concentrations found today, so not well constrained.
!  We calculate the present-day soil Hg distribution by adding a global mean
!  6.75 ng/g (=0.15 * 45 ng/g) according to present-day Hg deposition.
!  (eck, 11/13/08)
!
! !REVISION HISTORY:
!  30 Aug 2010 - N. Eckley, B. Corbitt - Initial version
!  (1 ) Added comments. (cdh, eds, 7/30/08)
!  (2 ) Now include light attenuation by the canopy after sunset. Emissions
!       change by < 1% in high-emission areas  (cdh, 8/13/2008)
!  (3 ) Removed FRCLND for consistency with other Hg emissions (cdh, 8/19/08)
!  2 June 2010 - C. Carouge  - Solve  
!  13 Aug 2010 - R. Yantosca - Added modifications for MERRA
!  25 Aug 2010 - R. Yantosca - Treat MERRA in same way as GEOS-5
!  26 Apr 2011 - J. Fisher   - Use MERRA land fraction information
!  12 Apr 2011 - J. Fisher   - Bug fixes, add missing code from Holmes 2010
!  08 Feb 2012 - R. Yantosca - Treat GEOS-5.7.x in the same way as MERRA
!  10 Feb 2012 - R. Yantosca - Extend #if statement for SOIL_EMIS_FAC in order
!                              to get the code to compile w/o error.
!  01 Mar 2012 - R. Yantosca - Now use GET_AREA_CM2(I,J,L) from grid_mod.F90
!  11 Apr 2012 - R. Yantosca - Replace lai_mod.F with modis_lai_mod.F90
!  09 Nov 2012 - M. Payer    - Replaced all met field arrays with State_Met
!                              derived type object
!  28 Nov 2012 - R. Yantosca - Replace SUNCOS with State_Met%SUNCOS
!  26 Sep 2013 - R. Yantosca - Renamed GEOS_57 Cpp switch to GEOS_FP
!EOP
!------------------------------------------------------------------------------
!BOC
! 
! !LOCAL VARIABLES:
!     
      LOGICAL             :: IS_SNOWFREE_LAND 
      INTEGER             :: I, J, JLOOP
      REAL(fp)            :: SOIL_EMIS, DIMLIGHT, TAUZ, LIGHTFRAC
      REAL(fp)            :: AREA_M2, DRYSOIL_HG
      REAL(fp)            :: SUNCOSVALUE
      REAL(fp)            :: FRAC_SNOWFREE_LAND 
!
! !DEFINED PARAMETERS:
!
      ! Preindustrial global mean soil Hg concentration, ng Hg /g dry soil
      REAL(fp), PARAMETER  :: DRYSOIL_PREIND_HG = 45e+0_fp

      ! Scaling factor for emissions, g soil /m2 /h
      ! (This parameter is beta in Eq 3 of Selin et al., GBC 2008.
      ! The value in paper is actually DRYSOIL_PREIND_HG * SOIL_EMIS_FAC 
      ! and the stated units are incorrect. The paper should have stated
      ! beta = 1.5D15 / 45D0 = 3.3D13)
      ! This parameter is tuned in the preindustrial simulation 
      ! so that total deposition to soil equals total emission from soil,
      ! while also requiring global mean soil Hg concentration of 45 ng/g 
#if defined (GRID4x5)
      REAL(fp), PARAMETER :: SOIL_EMIS_FAC = 1.6e-2_fp ! for sunlight function, yzh
#elif defined(GRID2x25)
      REAL(fp), PARAMETER :: SOIL_EMIS_FAC=1.6e-2_fp
#elif defined(GRID05x0666)
      !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ! lhu, 1/5/2012, add pre-defined SOIL_EMIS_FAC for 05x0667 simulation
      ! This is a non-physical value and would need to be changed for an
      ! actual mercury simulation
      REAL(fp), PARAMETER :: SOIL_EMIS_FAC = 1.6e-2_fp  ! yzh
      !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#elif defined(GRID025x03125)
      !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ! lhu, 1/5/2012, add pre-defined SOIL_EMIS_FAC for 05x0667 simulation
      ! This is a non-physical value and would need to be changed for an
      ! actual mercury simulation
      REAL(fp), PARAMETER :: SOIL_EMIS_FAC = -9.0e+99_fp
      !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#else  ! This sets a default value for GCM & ESMF COUPLING
      REAL(fp), PARAMETER :: SOIL_EMIS_FAC = 2.4e-2_fp*.5742e+0_fp ! for sunlight function
#endif

      !=================================================================
      ! SOILEMIS begins here!
      !=================================================================

!$OMP PARALLEL DO
!$OMP+DEFAULT( SHARED )
!$OMP+PRIVATE( I,          J,    SOIL_EMIS                       ) 
!$OMP+PRIVATE( DRYSOIL_HG, TAUZ, LIGHTFRAC, AREA_M2, SUNCOSVALUE ) 
!$OMP+PRIVATE( IS_SNOWFREE_LAND, FRAC_SNOWFREE_LAND              )
      DO J=1, JJPAR
      DO I=1, IIPAR
         
         ! Except in MERRA, assume whole box is land if conditions met
         ! (jaf, 4/26/11)
         FRAC_SNOWFREE_LAND = 1e+0_fp

#if   defined( MERRA ) || defined( GEOS_FP )
         FRAC_SNOWFREE_LAND = MAX( State_Met%FRLAND(I,J) - 
     &                             State_Met%FRSNO(I,J), 0e+0_fp )
         IS_SNOWFREE_LAND   = ( FRAC_SNOWFREE_LAND > 0e+0_fp )

#elif defined( GEOS_5 )
         ! GEOS5 snow height (water equivalent) in mm. (Docs wrongly say m)
         IS_SNOWFREE_LAND = ( IS_LAND( I, J, State_Met ) .AND.
     &                      ( State_Met%SNOMAS(I,J) < 10e+0_fp ))

#else
         ! GEOS1-4 snow heigt (water equivalent) in mm
         IS_SNOWFREE_LAND = ( IS_LAND( I, J, State_Met ) .AND.
     &                      ( State_Met%SNOW(I,J)   < 10e+0_fp ))
#endif

         IF ( IS_SNOWFREE_LAND ) THEN

            ! attenuate solar radiation based on function of leaf area index
            ! Jacob and Wofsy 1990 equations 8 & 9
            TAUZ = ISOLAI(I,J) * 0.5e+0_fp

            ! For very low and below-horizon solar zenith angles, use
            ! same attenuation as for SZA=85 degrees
            SUNCOSVALUE = MAX( State_Met%SUNCOS(I,J), 0.09e+0_fp )

            ! fraction of light reaching the surface is
            ! attenuated based on LAI
            LIGHTFRAC = EXP( -TAUZ / SUNCOSVALUE )

            ! Dry soil Hg concentration, ng Hg /g soil
            DRYSOIL_HG = DRYSOIL_PREIND_HG * EHg0_dist(I,J)           

            ! Soil emissions, ng /m2 /h
            SOIL_EMIS = 
     &           EXP( 0.0011 * State_Met%RADSWG(I,J) * LIGHTFRAC ) *
     &           DRYSOIL_HG * SOIL_EMIS_FAC
     
            ! Grid box surface area [m2]
            AREA_M2   = GET_AREA_M2( I, J, 1 )
 
            ! convert soilnat from ng /m2 /h -> kg /gridbox /s
            EHg0_so(I,J) = SOIL_EMIS * AREA_M2 * 1e-12_fp / 
     &                     ( 60e+0_fp * 60e+0_fp )

            ! Multiply by fractional land area
            EHg0_so(I,J) = EHg0_so(I,J) * FRAC_SNOWFREE_LAND

         ELSE

            ! no soil emissions from water and ice
            EHg0_so(I,J) = 0e+0_fp
        
         ENDIF
        
      ENDDO
      ENDDO
!$OMP END PARALLEL DO

      END SUBROUTINE SOILEMIS
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: snowpack_mercury_flux
!
! !DESCRIPTION: Subroutine SNOWPACK\_MERCURY\_FLUX calculates emission of 
!  Hg(0) from snow and ice. 
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE SNOWPACK_MERCURY_FLUX( FLUX, LHGSNOW, State_Met )
!
! !USES:
!
      USE DEPO_MERCURY_MOD,   ONLY : SNOW_HG
      USE GIGC_State_Met_Mod, ONLY : MetState
      USE TIME_MOD,           ONLY : GET_TS_EMIS
      USE TRACERID_MOD,       ONLY : N_Hg_CATS

      USE CMN_SIZE_MOD             ! Size parameters
!
! !INPUT PARAMETERS: 
!
      LOGICAL,        INTENT(IN)  :: LHGSNOW     ! Use Hg from snow?
      TYPE(MetState), INTENT(IN)  :: State_Met   ! Meteorology State object
!
! !OUTPUT PARAMETERS:
!
      REAL(fp),       INTENT(OUT) :: FLUX(IIPAR,JJPAR,N_Hg_CATS)  ! Hg0 flux
                                                                  !  [kg/s]
!
! !REMARKS:
!  Emissions are a linear function of Hg mass stored in the snowpack. The
!  Hg lifetime in snow is assumed to be 180 d when T< 270K and 7 d when T>270K
!
!     E = k * SNOW_HG     : k = 6D-8 if T<270K, 1.6D-6 otherwise
!
!  These time constants reflect the time scales of emission observed in the 
!  Arctic and in field studies. Holmes et al 2010
! 
! !REVISION HISTORY: 
!  15 Sep 2009 - C. Holmes, S. Carouge - Initial version
!  30 Aug 2010 - R. Yantosca - Added ProTex headers
!  12 Apr 2011 - J. Fisher   - Add missing code from Holmes 2010
!  09 Nov 2012 - M. Payer    - Replaced all met field arrays with State_Met
!                              derived type object
!  28 Nov 2012 - R. Yantosca - Replace SUNCOS with State_Met%SUNCOS
!  30 Mar 2015 - R. Yantosca - Remove obsolete variables
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER  :: I, J, NN, JLOOP
      REAL(fp) :: DTSRCE, SNOW_HG_NEW, K_EMIT

      !=================================================================
      ! SNOWPACK_MERCURY_FLUX begins here!
      !=================================================================

      ! Initialize
      FLUX = 0e+0_fp

      ! Return to calling program if snowpack model is disabled
      IF ( .NOT. LHGSNOW ) RETURN

      ! Emission timestep [s]
      DTSRCE = GET_TS_EMIS() * 60e+0_fp      

!$OMP PARALLEL DO 
!$OMP+DEFAULT( SHARED ) 
!$OMP+PRIVATE( I, J, NN, SNOW_HG_NEW, K_EMIT )
      DO J = 1, JJPAR
      DO I = 1, IIPAR

         ! Zero for safety's sake
         K_EMIT      = 0.0e+0+fp
         SNOW_HG_NEW = 0.0e+0_fp

         ! If the sun is set, then no emissions, go to next box
         IF ( State_Met%SUNCOS(I,J) < 0e+0_fp ) CYCLE 

         ! Decrease residence time to 1 week when T > -3C
         ! Update to Holmes et al. 2010 version (jaf, 4/11/11)
         ! Emit Hg(0) at a steady rate, based on 180 d residence
         ! time in snowpack, based on cycle observed at Alert 
         ! (e.g. Steffen et al. 2008)
         IF ( State_Met%T(I,J,1) > 270e+0_fp) THEN

            ! NOW USE 3 weeks for broader summer peak
            !K_EMIT = 1.6D-6
            K_EMIT = 5e-7_fp

         ELSE

            ! Emit Hg(0) at a steady rate, based on 180 d residence
            ! time in snowpack, based on cycle observed at Alert 
            ! (e.g. Steffen et al. 2008)
            K_EMIT = 6e-8_fp

         ENDIF

         ! Loop over # of categories 
         DO NN = 1, N_Hg_CATS

            ! Check if there is Hg that could be emitted
            IF ( SNOW_HG(I,J,NN) > 0e+0_fp ) THEN

               ! New mass of snow in Hg
               SNOW_HG_NEW = SNOW_HG(I,J,NN) * EXP( -K_EMIT * DTSRCE )

               ! Not less than zero
               FLUX(I,J,NN) = MAX( SNOW_HG(I,J,NN) - SNOW_HG_NEW,
     &                             0e+0_fp )

               ! Convert mass -> flux
               FLUX(I,J,NN) = FLUX(I,J,NN) / DTSRCE

               ! Save back into array
               SNOW_HG(I,J,NN) = SNOW_HG_NEW

            ENDIF

         ENDDO

      ENDDO
      ENDDO
!$OMP END PARALLEL DO
      
      END SUBROUTINE SNOWPACK_MERCURY_FLUX
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: gtmm_dr
!
! !DESCRIPTION: GTMM\_DR is a driver to call GTMM from GEOS-Chem.
!\\
!\\
! !INTERFACE: 
!
      SUBROUTINE GTMM_DR( am_I_Root, Input_Opt, State_Met, Hg0gtm, RC )
! 
! !USES:
!
      USE BPCH2_MOD
      USE DAO_MOD,            ONLY : IS_LAND
      USE DEPO_MERCURY_MOD,   ONLY : CHECK_DIMENSIONS
      USE DEPO_MERCURY_MOD,   ONLY : WD_Hg2, WD_HgP, DD_HgP, DD_Hg2
      USE DEPO_MERCURY_MOD,   ONLY : READ_GTMM_RESTART
      USE FILE_MOD,           ONLY : IOERROR
      USE GIGC_ErrCode_Mod
      USE GIGC_Input_Opt_Mod, ONLY : OptInput
      USE GIGC_State_Met_Mod, ONLY : MetState
      USE inquireMod,         ONLY : findFreeLun
      USE TIME_MOD,           ONLY : EXPAND_DATE, YMD_EXTRACT
      USE TIME_MOD,           ONLY : GET_NYMD, GET_NHMS
    
      USE CMN_SIZE_MOD             ! Size parameters
!
! !INPUT PARAMETERS: 
!
      LOGICAL,        INTENT(IN)  :: am_I_Root   ! Are we on the root CPU?
      TYPE(OptInput), INTENT(IN)  :: Input_Opt   ! Input Options object      
      TYPE(MetState), INTENT(IN)  :: State_Met   ! Meteorology State object
!
! !OUTPUT PARAMETERS: 
!
      ! Emission of Hg0 calculated by GTMM for the month [kg/s]
      REAL(fp),         INTENT(OUT) :: Hg0gtm(IIPAR, JJPAR) 

      ! Success or failure?
      INTEGER,        INTENT(OUT) :: RC
! 
! !REMARKS:
!  ##########################################################################
!  #####    NOTE: BINARY PUNCH INPUT IS BEING PHASED OUT.  THIS DATA    #####
!  #####    WILL EVENTUALLY BE READ IN FROM netCDF FILES VIA HEMCO!     #####
!  #####       -- Bob Yantosca (05 Mar 2015)                            #####
!  ##########################################################################
!
! !REVISION HISTORY:
!  15 Sep 2009 - C. Carouge  - Initial version
!  03 Aug 2012 - R. Yantosca - Move calls to findFreeLUN out of DEVEL block
!  05 Mar 2015 - R. Yantosca - Now add Input_Opt%RES_DIR to data path
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER   :: YEAR    ! Current year 
      INTEGER   :: MONTH   ! Current month
      INTEGER   :: DAY
      INTEGER   :: NYMD, NHMS

      REAL(fp), DIMENSION(IIPAR, JJPAR)   :: TSURF    ! Ground temperature
      REAL(fp), DIMENSION(IIPAR, JJPAR)   :: PRECIP   ! Total precipitation
                                                    ! for the month
      REAL(fp), DIMENSION(IIPAR, JJPAR)   :: SOLAR_W  ! Solar radiation for 
                                                    ! the month

      REAL*4, DIMENSION(IIPAR, JJPAR)   :: TRACER   ! Temporary array

      ! Monthly average deposition arrays
      REAL(fp), DIMENSION(IIPAR, JJPAR)   :: Hg0mth_dry
      REAL(fp), DIMENSION(IIPAR, JJPAR)   :: Hg2mth_dry
      REAL(fp), DIMENSION(IIPAR, JJPAR)   :: Hg2mth_wet

      INTEGER               :: IOS, I, J, L, IU_FILE

      CHARACTER(LEN=255)    :: FILENAME

      ! For binary punch file, version 2.0
      INTEGER               :: NI,        NJ,      NL
      INTEGER               :: IFIRST,    JFIRST,  LFIRST
      INTEGER               :: HALFPOLAR, CENTER180
      INTEGER               :: NTRACER,   NSKIP
      REAL*4                :: LONRES,    LATRES
      REAL(fp)                :: ZTAU0,     ZTAU1
      CHARACTER(LEN=20)     :: MODELNAME
      CHARACTER(LEN=40)     :: CATEGORY
      CHARACTER(LEN=40)     :: UNIT     
      CHARACTER(LEN=40)     :: RESERVED


      !=================================================================
      ! GTMM_DR begins here!
      !=================================================================

      ! Assume success
      RC         = GIGC_SUCCESS

      ! Initialise arrays
      NYMD       = GET_NYMD()
      NHMS       = GET_NHMS()

      TSURF      = 0e+0_fp
      PRECIP     = 0e+0_fp
      SOLAR_W    = 0e+0_fp
      Hg0gtm     = 0e+0_fp

      ! Reset deposition arrays.
      Hg0mth_dry = 0e+0_fp
      Hg2mth_dry = 0e+0_fp
      Hg2mth_wet = 0e+0_fp

      CALL YMD_EXTRACT( NYMD, YEAR, MONTH, DAY )

      !=================================================================
      ! Read monthly meteorology fields
      !=================================================================

      ! Find a free file LUN
      IU_FILE  = findFreeLun()

      ! File name
      FILENAME = TRIM( Input_Opt%DATA_DIR ) //
     &           TRIM( Input_Opt%RES_DIR  ) // 'mercury_201007/' //
     &           'mean_metfields/' // GET_NAME_EXT() // 
     &           '/mean_YYYYMM.bpch'

      ! Replace YYYY, MM, DD, HH tokens in FILENAME w/ actual values
      CALL EXPAND_DATE( FILENAME, NYMD, NHMS )
    
      ! Echo some input to the screen
      WRITE( 6, '(a)' ) REPEAT( '=', 79 )
      WRITE( 6, 100   ) 
      WRITE( 6, 110   ) TRIM( FILENAME ), IU_FILE
 100  FORMAT( 'G T M M  H g   M E T   F I L E   I N P U T' )
 110  FORMAT( /, 'GTMM_DR: Reading ', a, ' on unit ', i6  )

      ! Open the binary punch file for input
      CALL OPEN_BPCH2_FOR_READ( IU_FILE, FILENAME )
    
      !-----------------------------------------------------------------
      ! Read concentrations -- store in the TRACER array
      !-----------------------------------------------------------------
      DO 
         READ( IU_FILE, IOSTAT=IOS )                              
     &        MODELNAME, LONRES, LATRES, HALFPOLAR, CENTER180
       
         ! IOS < 0 is end-of-file, so exit
         IF ( IOS < 0 ) EXIT
       
         ! IOS > 0 is a real I/O error -- print error message
         IF ( IOS > 0 ) CALL IOERROR( IOS, IU_FILE, 'rd_gtmm_dr:1' )
       
         READ( IU_FILE, IOSTAT=IOS )                               
     &        CATEGORY, NTRACER,  UNIT, ZTAU0,  ZTAU1,  RESERVED,  
     &        NI,       NJ,       NL,   IFIRST, JFIRST, LFIRST,    
     &        NSKIP
       
         IF ( IOS /= 0 ) CALL IOERROR( IOS, IU_FILE, 'rd_gtmm_dr:2' )
       
         READ( IU_FILE, IOSTAT=IOS )                               
     &        ( ( TRACER(I,J), I=1,NI ), J=1,NJ )
       
         IF ( IOS /= 0 ) CALL IOERROR( IOS, IU_FILE, 'rd_gtmm_dr:3' )
       
         !--------------------------------------------------------------
         ! Assign data from the TRACER array to the arrays.
         !--------------------------------------------------------------
       
         ! Process dry deposition data 
         IF ( CATEGORY(1:8) == 'GMAO-2D' ) THEN 
          
            ! Make sure array dimensions are of global size
            ! (NI=IIPAR; NJ=JJPAR, NL=1), or stop the run
            CALL CHECK_DIMENSIONS( NI, NJ, NL )
          
            ! Save into arrays
            IF ( NTRACER == 54 .OR. NTRACER == 59 ) THEN
             
               !----------
               ! Surface temperature
               !----------
             
               ! Store surface temperature in TSURF array
               TSURF(:,:)   = TRACER(:,JJPAR:1:-1)
             
            ELSE IF ( NTRACER == 26 .OR. NTRACER == 29 ) THEN
             
               !----------
               ! Total precipitation
               !----------
             
               ! Store precipitation in PRECIP array
               PRECIP(:,:)   = TRACER(:,JJPAR:1:-1)
             
            ELSE IF ( NTRACER == 37 .OR. NTRACER == 51 ) THEN

               !----------
               ! Solar radiation
               !----------

               ! Store solar radiation in SOLAR_W array
               SOLAR_W(:,:) = TRACER(:,JJPAR:1:-1)

            ENDIF
         ENDIF
       
      ENDDO
    
      ! Close file
      CLOSE( IU_FILE )      
    
      !=================================================================
      ! Read GTMM restart file to get data from previous month
      !=================================================================
      CALL READ_GTMM_RESTART( am_I_Root,  Input_Opt,  NYMD,      
     &                        NHMS,       Hg0mth_dry, Hg2mth_dry, 
     &                        Hg2mth_wet, RC  )

      !=================================================================
      ! Call GTMM model
      !=================================================================
#if defined( GTMM_Hg )
      CALL GTMM_coupled(YEAR                    , MONTH  , 
     &                  Hg0mth_dry(:,JJPAR:1:-1), 
     &                  Hg2mth_dry(:,JJPAR:1:-1), 
     &                  Hg2mth_wet(:,JJPAR:1:-1), TSURF  , PRECIP, 
     &                  SOLAR_W                  , Hg0gtm(:,JJPAR:1:-1)
     &                  )
#endif

      DO J = 1, JJPAR
      DO I = 1, IIPAR
         IF ( .NOT.(IS_LAND(I, J, State_Met )) ) Hg0gtm(I,J) = 0e+0_fp
      ENDDO
      ENDDO

      END SUBROUTINE GTMM_DR      
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: init_land_mercury
!
! !DESCRIPTION: Subroutine INIT\_LAND\_MERCURY allocates and zeroes all 
!  module arrays.
!\\
!\\
! !INTERFACE: 
!
      SUBROUTINE INIT_LAND_MERCURY( am_I_Root, Input_Opt, RC )
! 
! !USES:
!
      USE CMN_SIZE_MOD
      USE ERROR_MOD,          ONLY : ALLOC_ERR
      USE GIGC_ErrCode_Mod
      USE GIGC_Input_Opt_Mod, ONLY : OptInput
!
! !INPUT PARAMETERS:
!
      LOGICAL,        INTENT(IN)    :: am_I_Root   ! Are we on the root CPU?
      TYPE(OptInput), INTENT(IN)    :: Input_Opt   ! Input Options object
!
! !OUTPUT PARAMETERS:
!
      INTEGER,        INTENT(OUT)   :: RC          ! Success or failure?
!
! !REVISION HISTORY:
!  14 Sep 2009 - C. Carouge  - Initial version
!  23 Jun 2014 - R. Yantosca - Now accept am_I_Root, Input_Opt, RC
!EOP
!------------------------------------------------------------------------------
!BOC
      !=================================================================
      ! INIT_MERCURY begins here!
      !=================================================================

      END SUBROUTINE INIT_LAND_MERCURY
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: cleanup_land_mercury
!
! !DESCRIPTION: Subroutine CLEANUP\_LAND\_MERCURY deallocates all module 
!  arrays. 
!\\
!\\
! !INTERFACE: 
!
      SUBROUTINE CLEANUP_LAND_MERCURY
! 
! !REVISION HISTORY:
!  14 Sep 2009 - C. Carouge  - Initial version
!EOP
!------------------------------------------------------------------------------
!BOC

      END SUBROUTINE CLEANUP_LAND_MERCURY
!EOC
      END MODULE LAND_MERCURY_MOD
